import requests
from bs4 import BeautifulSoup


class EndWayApi:
    def __init__(self, xf_user: str) -> None:
        self.session = requests.session()
        self.cookies = {"xf_user": xf_user}
        self.xfToken = self.get_token()

    def get_token(self) -> str:
        """
        Getting a csrf token for later use in requests

        :return: csrf token
        :rtype: str
        """
        response = self.session.get("https://endway.su/", cookies=self.cookies).text
        soup = BeautifulSoup(response, "html.parser")
        csrf = soup.find("html").get("data-csrf")
        return csrf

    def post_thread(self, section_id: int, title: str, message: str) -> dict:
        """
        Creating a post

        :param section_id: The number of the partition in which the thread will be created
        :type section_id: int

        :param title: Topic title
        :type title: str

        :param message: Message
        :type message: str

        :return: Response from the server
        :rtype: dict
        """
        data = {
            "_xfToken": self.xfToken,
            "title": title,
            "message_html": message,
            "_xfResponseType": "json"
        }
        response = self.session.post(f"https://endway.su/forums/{section_id}/post-thread", cookies=self.cookies,
                                     data=data).json()
        return response

    def add_reply(self, thread_id: int, message: str) -> dict:
        """
        Sending a message to a thread

        :param thread_id: The thread number to which the message will be sent
        :type thread_id: int

        :param message: Message
        :type message: str

        :return: Response from the server
        :rtype: dict
        """
        data = {
            "_xfToken": self.xfToken,
            "message_html": message,
            "_xfResponseType": "json"
        }
        response = self.session.post(f"https://endway.su/threads/{thread_id}/add-reply", cookies=self.cookies,
                                     data=data).json()
        return response

    def member_post(self, member_id: int or str, message: str) -> dict:
        """
        Creating a post on a user's page

        :param member_id: Either a permanent user id or a short link. If permanent id, then type int, if short link,
        then type string
        :type member_id: int or str

        :param message: Message
        :type message: str

        :return: Response from the server
        :rtype: dict
        """
        if type(member_id) is int:
            url = f"https://endway.su/members/{member_id}/post"
        else:
            url = f"https://endway.su/{member_id}/post"

        data = {
            "_xfToken": self.xfToken,
            "message_html": message,
            "_xfResponseType": "json"
        }
        response = self.session.post(url, cookies=self.cookies, data=data).json()
        return response

    def add_comment(self, post_id: int, message: str) -> dict:
        """
        Creating a comment under a post on a user's page

        :param post_id: The number of the message on the user's page
        (I do not know how to get it without using DevTools)
        :type post_id: int

        :param message: Message
        :type message: str

        :return: Response from the server
        :rtype: dict
        """
        data = {
            "_xfToken": self.xfToken,
            "message_html": message,
            "_xfResponseType": "json"
        }
        response = self.session.post(f"https://endway.su/profile-posts/{post_id}/add-comment", cookies=self.cookies,
                                     data=data).json()
        return response

    # Changing Account Information
    def account_details(self, short_link: str = None, location: str = None, website: str = None,
                        profilebackground: str = None, about_html: str = None, telegram: str = None, vk: str = None,
                        steam: str = None, discord: str = None, github: str = None) -> dict:
        data = {
            "_xfToken": self.xfToken,
            "short_link": short_link,
            "profile[location]": location,
            "profile[website]": website,
            "custom_fields[profilebackground]": profilebackground,
            "about_html": about_html,
            "custom_fields[telegram]": telegram,
            "custom_fields[vk]": vk,
            "custom_fields[Steam]": steam,
            "custom_fields[discord]": discord,
            "custom_fields[github]": github,
            "_xfResponseType": "json"
        }
        response = self.session.post(f"https://endway.su/account/account-details", cookies=self.cookies,
                                     data=data).json()
        return response

    def security(self, old_password: str, new_password: str) -> dict:
        """
        Changing the account password

        :param old_password: Old account password
        :type old_password: str

        :param new_password: New password for the account
        :type new_password: str

        :return: Response from the server
        :rtype: dict
        """

        data = {
            "_xfToken": self.xfToken,
            "old_password": old_password,
            "password": new_password,
            "password_confirm": new_password,
            "_xfResponseType": "json"
        }
        response = self.session.post(f"https://endway.su/account/security", cookies=self.cookies,
                                     data=data).json()
        return response

    def signature(self, signature_html: str) -> dict:
        """
        Changing the signature

        :param signature_html: New signature
        :type signature_html: str

        :return: Response from the server
        :rtype: dict
        """
        data = {
            "_xfToken": self.xfToken,
            "signature_html": signature_html,
            "_xfResponseType": "json"
        }
        response = self.session.post(f"https://endway.su/account/security", cookies=self.cookies,
                                     data=data).json()
        return response
